@@ -2,7 +2,6 @@ import os
 import sys
 import typing
 
-import asn1tools
 import wx
 import wx.svg
 
@@ -10,37 +9,40 @@ import asn1editor
 from asn1editor.ASN1SpecHandler import ASN1SpecHandler
 from asn1editor.Plugin import Plugin
 from asn1editor.PluginInterface import PluginInterface
-from asn1editor.wxPython import Environment
+from asn1editor.wxPython import Environment, Resources
 from asn1editor.wxPython import WxPythonViewFactory
 from asn1editor.wxPython.FilePickerHandler import FilePickerHandler
-from asn1editor.wxPython.Resources import resource_path
+from asn1editor.wxPython.ImageList import ImageList
+from asn1editor.wxPython.MenuHandler import MenuHandler
+from asn1editor.wxPython.SingleFileDropTarget import SingleFileDropTarget
 from asn1editor.wxPython.Styler import Styler
+from asn1editor.wxPython.TreeView import TreeView
+from asn1editor.wxPython.WxPythonViews import WxPythonView
 
 
 class MainWindow(wx.Frame, PluginInterface):
     def __init__(self, plugins: typing.Optional[typing.List[Plugin]] = None, title=f'ASN.1 editor {asn1editor.__version__}'):
-        super(MainWindow, self).__init__(None, title=title, size=(500, 800))
+        super(MainWindow, self).__init__(None, title=title)
 
         Environment.load()
+        Resources.image_list = ImageList()
 
-        self.__plugins = plugins
-        if self.__plugins is not None:
-            for plugin in self.__plugins:
+        if plugins is not None:
+            for plugin in plugins:
                 plugin.connect(self)
 
+        self._status_bar = self.CreateStatusBar()
+
+        self._menu_handler = MenuHandler(self, plugins)
+
+        self._menu_handler.build(self.load_spec, self.load_data_from_file, self.save_data_to_file, self._structure_changed)
+        self.Bind(wx.EVT_CLOSE, self.close)
+
         self.SetSize(wx.Size(Environment.settings.get('size', (500, 800))))
         self.Maximize(Environment.settings.get('maximized', True))
         self.SetPosition(wx.Point(Environment.settings.get('position', (0, 0))))
+        self._menu_handler.view_select.selected = Environment.settings.get('view', 1)
 
-        self.__main_panel = wx.ScrolledWindow(self, style=wx.HSCROLL | wx.VSCROLL)
-        self.__main_panel.SetScrollbars(15, 15, 50, 50)
-        sizer = wx.BoxSizer(wx.VERTICAL)
-        self.__main_panel.SetSizer(sizer)
-        self.__main_panel.SetAutoLayout(True)
-
-        self._status_bar = self.CreateStatusBar()
-
-        self.__create_menu()
         self.__asn1_handler = None
 
         self.__model = None
@@ -48,93 +50,25 @@ class MainWindow(wx.Frame, PluginInterface):
         self.__controller = None
         self.__type_name = None
         self.__file_name = None
+        self.__tree_view: typing.Optional[TreeView] = None
+        self.__content_panel: typing.Optional[wx.ScrolledWindow] = None
 
         self.__progress_window: typing.Optional[wx.ProgressDialog] = None
 
-        self.bind_events()
+        self.SetDropTarget(SingleFileDropTarget(self.__file_dropped))
+        self.SetAutoLayout(True)
+        self.SetSizer(wx.BoxSizer(wx.HORIZONTAL))
 
         # noinspection SpellCheckingInspection
         sys.excepthook = self.__exception_handler
 
-    def __create_menu(self):
-        menu_bar = wx.MenuBar()
-        file_menu = wx.Menu()
-        self.__load_spec_item: wx.MenuItem = file_menu.Append(wx.ID_ANY, 'Open ASN.1 specification')
-        # noinspection PyArgumentList
-        image: wx.svg.SVGimage = wx.svg.SVGimage.CreateFromFile(resource_path('icons/open.svg'))
-        self.__load_spec_item.SetBitmap(image.ConvertToBitmap(width=16, height=16))
-        file_menu.AppendSeparator()
-        self.__load_data_item: wx.MenuItem = file_menu.Append(wx.ID_OPEN, 'Load encoded data')
-        # noinspection PyArgumentList
-        image: wx.svg.SVGimage = wx.svg.SVGimage.CreateFromFile(resource_path('icons/load_encoded.svg'))
-        self.__load_data_item.SetBitmap(image.ConvertToBitmap(width=16, height=16))
-        self.__load_data_item.Enable(False)
-        self.__save_data_item: wx.MenuItem = file_menu.Append(wx.ID_SAVE, 'Save encoded data')
-        # noinspection PyArgumentList
-        image: wx.svg.SVGimage = wx.svg.SVGimage.CreateFromFile(resource_path('icons/save_encoded.svg'))
-        self.__save_data_item.SetBitmap(image.ConvertToBitmap(width=16, height=16))
-        self.__save_data_item.Enable(False)
-        file_menu.AppendSeparator()
-        self.__exit_item = file_menu.Append(wx.ID_EXIT, 'Exit', 'Exit application')
-        # noinspection PyArgumentList
-        image: wx.svg.SVGimage = wx.svg.SVGimage.CreateFromFile(resource_path('icons/exit.svg'))
-        self.__exit_item.SetBitmap(image.ConvertToBitmap(width=16, height=16))
-        menu_bar.Append(file_menu, '&File')
+        WxPythonView.structure_changed = None
 
-        if self.__plugins is not None:
-
-            for plugin_index, plugin in enumerate(self.__plugins):
-                plugin_menu = wx.Menu()
-                menus = plugin.get_menus()
-                for i, menu in enumerate(menus):
-                    if not len(menu[0]):
-                        plugin_menu.AppendSeparator()
-                    else:
-                        menu_item: wx.MenuItem = plugin_menu.Append(plugin_index * 1000 + i, menu[0])
-                        if menu[1] is not None:
-                            self.Bind(wx.EVT_MENU, self.__plugin_menu_event, menu_item)
-                        else:
-                            menu_item.Enable(False)
-
-                menu_bar.Append(plugin_menu, plugin.get_name())
-
-        help_menu = wx.Menu()
-        about_item = help_menu.Append(wx.ID_ABOUT, 'About')
-        self.Bind(wx.EVT_MENU, self.__about_item_event, about_item)
-        menu_bar.Append(help_menu, '&Help')
-
-        self.SetMenuBar(menu_bar)
-
-    def __plugin_menu_event(self, e):
-        plugin_index = e.GetId() // 1000
-        self.__plugins[plugin_index].get_menus()[e.GetId() % 1000][1]()
-
-    def bind_events(self):
-        def schema_dialog_constructor() -> wx.FileDialog:
-            return wx.FileDialog(self, "ASN.1 schema", wildcard="ASN.1 files (*.asn)|*.asn",
-                                 style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_MULTIPLE)
-
-        def data_load_dialog_constructor() -> wx.FileDialog:
-            extensions = ';'.join(ASN1SpecHandler.get_extensions())
-            return wx.FileDialog(self, "ASN.1 encoded file", wildcard=f"ASN.1 encoded ({extensions})|{extensions}",
-                                 style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
-
-        def data_save_dialog_constructor() -> wx.FileDialog:
-            extensions = ';'.join(ASN1SpecHandler.get_extensions())
-            return wx.FileDialog(self, "ASN.1 encoded file", wildcard=f"ASN.1 encoded ({extensions})|{extensions}", style=wx.FD_SAVE)
-
-        self.Bind(wx.EVT_MENU, self.exit, self.__exit_item)
-
-        picker = FilePickerHandler(schema_dialog_constructor, self.load_spec)
-        self.Bind(wx.EVT_MENU, picker.on_menu_click, self.__load_spec_item)
-
-        picker = FilePickerHandler(data_load_dialog_constructor, self.load_data_from_file)
-        self.Bind(wx.EVT_MENU, picker.on_menu_click, self.__load_data_item)
-
-        picker = FilePickerHandler(data_save_dialog_constructor, self.save_data_to_file, True)
-        self.Bind(wx.EVT_MENU, picker.on_menu_click, self.__save_data_item)
-
-        self.Bind(wx.EVT_CLOSE, self.close, self)
+    def __file_dropped(self, file_name: str):
+        if self.__asn1_handler is not None:
+            self.load_data_from_file(file_name)
+        else:
+            self.load_spec(file_name)
 
     def load_spec(self, file_name: str, type_name: typing.Optional[str] = None):
         # Spec file loaded, compile it to show a selection of type names
@@ -151,31 +85,71 @@ class MainWindow(wx.Frame, PluginInterface):
                 dialog.Destroy()
         else:
             self.__type_name = type_name
+
         if self.__type_name is not None:
             self._status_bar.SetStatusText(f'Loaded {file_name}')
             self.__file_name = file_name
 
-            styler = Styler(os.path.splitext(file_name)[0] + '.style')
-            view_factory = WxPythonViewFactory.WxPythonViewFactory(self.__main_panel, styler)
-            view_factory.freeze()
-
             if self.__view is not None:
-                self.__view.destroy()
+                self.__view.realize().destroy()
+                self.__content_panel.Destroy()
+            if self.__tree_view is not None:
+                self.__tree_view.destroy()
+
+            styler = Styler(os.path.splitext(file_name)[0] + '.style')
+
+            WxPythonView.structure_changed = lambda x: None
+
+            self.__content_panel = wx.ScrolledWindow(self, style=wx.HSCROLL | wx.VSCROLL)
+            self.__content_panel.SetScrollbars(15, 15, 50, 50)
+            self.__content_panel.SetAutoLayout(True)
+            self.__content_panel.SetSizer(wx.BoxSizer(wx.VERTICAL))
+
+            view_factory = WxPythonViewFactory.WxPythonViewFactory(self.__content_panel, styler)
+
+            self.Freeze()
 
             self.__view, self.__controller = self.__asn1_handler.create_mvc_for_type(self.__type_name, view_factory)
-            sizer: wx.Sizer = self.__main_panel.GetSizer()
-            sizer.Clear()
-            sizer.Add(self.__view.realize(), 0, wx.ALL | wx.EXPAND, 5)
-            sizer.Layout()
+            self.__tree_view = TreeView(self, self.__content_panel, self.__type_name)
 
-            self.__main_panel.SetSizer(sizer)
-            self.__main_panel.FitInside()
-            self.__main_panel.AdjustScrollbars()
+            self.Thaw()
 
-            view_factory.thaw()
+            WxPythonView.structure_changed = self._structure_changed
+            self._structure_changed()
 
-            self.__load_data_item.Enable(True)
-            self.__save_data_item.Enable(True)
+            self._menu_handler.enable()
+
+    def _structure_changed(self):
+        self.Freeze()
+
+        sizer = self.GetSizer()
+        sizer.Clear()
+        content_panel_sizer: wx.Sizer = self.__content_panel.GetSizer()
+        content_panel_sizer.Clear()
+
+        if self._menu_handler.view_select.selected == self._menu_handler.view_select.TREE:
+
+            tree_ctrl = self.__tree_view.get_ctrl(self.__view.realize())
+
+            sizer.Add(tree_ctrl, proportion=1, flag=wx.ALL | wx.EXPAND)
+            sizer.Add(self.__content_panel, proportion=2, flag=wx.ALL | wx.EXPAND)
+        else:
+            self.__tree_view.hide()
+
+            self.__view.realize().set_visible(True, recursive=True)
+            content_sizer = self.__view.realize().get_sizer(recursive=True)
+            content_panel_sizer.Add(content_sizer, flag=wx.ALL | wx.EXPAND, border=5)
+
+            self.__content_panel.SetSizerAndFit(content_panel_sizer)
+
+            sizer.Add(self.__content_panel, flag=wx.ALL | wx.EXPAND)
+
+        self.SetSizer(sizer)
+
+        self.Refresh()
+        self.PostSizeEvent()
+
+        self.Thaw()
 
     def load_data_from_file(self, file_name: str):
         self.__controller.model_to_view(self.__asn1_handler.load_data_file(file_name))
@@ -278,31 +252,12 @@ class MainWindow(wx.Frame, PluginInterface):
         self._status_bar.SetStatusText(f'Error: {value}')
         Environment.log_error(exception_str)
 
-    # noinspection PyUnusedLocal
-    def __about_item_event(self, e):
-        del e
-        dialog = wx.MessageDialog(self, f'''asn1editor {asn1editor.__version__}
-
-Published under MIT License
-
-Copyright (c) 2020 Florian Fetz
-https://github.com/Futsch1/asn1editor
-
-Based on eerimoq's asn1tools, used in {asn1tools.version.__version__}
-https://github.com/eerimoq/asn1tools
-''', style=wx.ICON_INFORMATION | wx.OK, caption='About')
-        dialog.ShowModal()
-
-    # noinspection PyUnusedLocal
-    def exit(self, e: wx.Event):
-        del e
-        self.Close()
-
     # noinspection PyUnusedLocal
     def close(self, e: wx.Event):
         Environment.settings['size'] = (self.GetSize().Get())
         Environment.settings['maximized'] = self.IsMaximized()
         Environment.settings['position'] = self.GetPosition().Get()
+        Environment.settings['view'] = self._menu_handler.view_select.selected
 
         Environment.save()
 
